<?php
/**
 * @version 2.0.1
 * @package moseasymedia
 * @copyright Copyright &copy; 2008, Brilaps LLC
 * @author Ozgur Cem Sen
 * @email code@brilaps.com
 * @link http://brilaps.com || http://wiki.brilaps.com
 * @license http://www.opensource.org/licenses/gpl-license.php GNU/GPL v.2 .
 */
/**
 * moseasymedia is inspired by mosmodule(http://ongetc.com/) with some extra kick from mosmedia(http://www.ag-solutions.net/)
 */


/**
 * 1.0.4 change. Supported plugins/players removed from the configuration options. There was really no need.
 */
global $SUPPORTED_MEDIA_PLAYERS;

$SUPPORTED_MEDIA_PLAYERS = array(
    "WindowsMediaPlayerVideo", 
    "WindowsMediaPlayerAudio", 
    "Flash", 
    "Shockwave", 
    "Quicktime", 
    "RealPlayer",
    "DivX",
    "JWFLVPlayer",
    "JWMediaPlayer",
    "JWMP3Player",
    "XSPFPlayer"
    );

/**
 * <p>moseasymediaPlayerEngine class acts as a player-specific-HTMLizer-engine to generate proper HTML output for supported media players.</p>
 * <p>Once the player type is determined, it's corresposing handler function will be called.</p>
 * <p>For future extensibility and as-of-now-clarity those functions are implemented in seperate files.</p>
 * <br/>i.e. if "Quicktime" player is associated with .mov files(in parameters),  moseasymediaQuicktime(...) function will be called from moseasymediaQuicktime.php file.
 * <ul>
 *    <li>Assume that we have a "BlahBlah" player which handles .blah files. This engine will call moseasymediaBlahBlah() function in moseasymediaBlahBlah.php file to generate HTML output.</li>
 *    <li>$SUPPORTED_MEDIA_PLAYERS  variable of the moseasymedia need to contain "BlahBlah" and set of params for "BlahBlah" required (at the least "ExtentionsBlahBlah" param)</li>
 *    <li>All that can be done reinstalling with an updated moseasymedia.xml file. Or updating the "params" field of the "#__mambots" table... (this almost sounds like a 'do not try this at home' :) )</li>
 * </ul>
 */
class moseasymediaPlayerEngine {

	var $moseasymediaparams = null;
	var $mediaAttribs = Array();
	var $mediaPlayer = "";

	var $DEFAULT_PLAYER = 'Flash';


	/**
	 * Constructor: Determines the media type and it's associated player, also do some constructor stuff.
	 * @param string $mediaAttribs URL or absolute or relative path to the media file and possible overridable attributes such as width height etc...
	 * @param object $moseasymediaparams mambot params for moseasymedia
	 */
	function moseasymediaPlayerEngine($mediaAttribs, $moseasymediaparams) {

		$this->mediaAttribs = $mediaAttribs;
		$this->moseasymediaparams = $moseasymediaparams;
		if (isset($mediaAttribs["player"]))
		$this->mediaPlayer = $mediaAttribs["player"];
		else
		$this->mediaPlayer = moseasymediaPlayerEngine::determineMediaPlayer($mediaAttribs["media"], $this->moseasymediaparams);

		foreach ($mediaAttribs as $attrib=>$value) {
			$botAttrib = $attrib.$this->mediaPlayer;
			//override the value...
			$this->moseasymediaparams->set($botAttrib, $value);
			
			//override the popup stuff here.  ocsmem_Popup_
			if (strpos($attrib, 'popup') !== false ) {
				$popupAttrib = 'ocsmem_Popup_'.$attrib;
				$this->moseasymediaparams->set($popupAttrib, $value);
			}
		}
	}

	/**
	 * Function: Detemines which player to use to HTMLize embeddation (what the heck is that !) .
	 * @param string $mediaFile URL or absolute or relative path to the media file
	 * @param object $moseasymediaparams mambot params for moseasymedia
	 * @return string media player to handle the HTMLization
	 *
	 * @ignore This seems like a static function-at least for now- incase I need to use it elsewhere.
	 * The class is using a media attributes array to override some mambot config attributes. This fuction does not deal with that.
	 * It would be wiser not to use this function externally, and let the class do the job. Otherwords, the following code is a better practice to handle the media.
	 *$mediaPlayerEngine = new moseasymediaPlayerEngine($mediaAttribs, $moseasymediaparams);
	 *$text = $mediaPlayerEngine->embed( );
	 */
	function determineMediaPlayer ($mediaPath, $moseasymediaparams){
		 
		global $SUPPORTED_MEDIA_PLAYERS;

		$filesplit = explode( '.', $mediaPath );
		$filetype = $filesplit[count($filesplit) - 1];

		//statement above may end up with a filetype com/v/BvzlZuWrJNw for video sharing websites.
		//probably the easiest check would be a / check.
		if (strpos($filetype, '/') == true)
		$filetype = '';

		//file type should suffice to determine the media type,
		//but not necessarily for the youtube, google, myspace videos
		//on top of all that, for example; we may want to play MPEG files in RealPlayer.
		for ($i = 0; $i < count($SUPPORTED_MEDIA_PLAYERS); $i++) {
			$player = trim($SUPPORTED_MEDIA_PLAYERS[$i]);
			if ($filetype != '') {
				$extensionParam = 'ocsmem_Extensions'.$player;
				if( eregi($filetype, $moseasymediaparams->get($extensionParam)) != false )
				return $player;
			}
			else {
				$mediaSharingWebsites = 'ocsmem_MediaSharingWebsitesUsing'.$player;
				$mediaSharingWebsitesUsingPlugin = explode(',',$moseasymediaparams->get($mediaSharingWebsites));
				//I really wanted to use regex in here, just went lazy.
				if ($mediaSharingWebsitesUsingPlugin[0] != NULL)
				foreach ($mediaSharingWebsitesUsingPlugin as $site) {
					$modpath = str_replace('/',' ',$mediaPath);
					$modpath = str_replace('.',' ',$mediaPath);
					$modsite = str_replace('.',' ',$site);
					if( eregi ($modsite, $modpath) != false)
					return $player;
				}
			}
		}

		//oh well, worst come to worst, try flash
		return $this->DEFAULT_PLAYER;
	}


	/**
	 * Function: Finally the plugin specific HTML is about to be generated.
	 *
	 * One of the following functions (see the link tags) will be called from their respective php files.
	 * @link moseasymediaFlash()
	 * @link moseasymediaQuicktime()
	 * @link moseasymediaRealPlayer()
	 * @link moseasymediaShockwave()
	 * @link moseasymediaWindowsMediaPlayerVideo()
	 * @link moseasymediaWindowsMediaPlayerAudio()
	 * @link moseasymediaXSPFPlayer()
	 * @link moseasymediaJWFLVPlayer()
	 * @link moseasymediaJWMediaPlayer()
	 * @link moseasymediaJWMP3Player()
	 * @link moseasymediaDivX()
	 *
	 * @return string HTML generated in order to emdeb the media file in the content.
	 */
	function embed() {

		global $mosConfig_absolute_path, $MOSEASYMEDIA_ABSOLUTE_PATH;
		$playerHandlerFileName = 'moseasymedia'.$this->mediaPlayer.'.php';
		if (file_exists($MOSEASYMEDIA_ABSOLUTE_PATH.$playerHandlerFileName) == true) {
			$playerHandlerFunction = 'moseasymedia'.$this->mediaPlayer;
			$playerHandlerClass = 'c'.$playerHandlerFunction;
			/**
			 * weird class instantiation, but gotta love php
			 */
			require_once($MOSEASYMEDIA_ABSOLUTE_PATH.$playerHandlerFileName);
			$embedClass = new $playerHandlerClass($this->mediaPlayer, $this->mediaAttribs["media"], $this->moseasymediaparams);
			
			return $embedClass->$playerHandlerFunction();
		}
		else {
			return "<div><strong>".$this->mediaPlayer."</strong> is not supported by moseasymedia. You probably made a typo ;) </div>";
		}
	}



	/**
	 * Function: Generates a key-value pair array from the valid attribute array with bot params
	 * @param string $player Name of the player
	 * @param array $attributes valid attributes for the OBJECT PARAM or EMBED code
	 * @param object $moseasymediaparams mambot params for moseasymedia
	 * @return string media player to handle the HTMLization
	 *
	 */
	function generateAttributeValuesArray($player, $attributes, $moseasymediaparams, $boolAsString = FALSE) {

		$attribute_values = array();
		foreach ($attributes as $attrib) {
			$attribX = $attrib.$player;
			$value = $moseasymediaparams->get($attribX);
			//this is very bizzare but required
			if (($value == "'true'") && ($boolAsString))
			$value = 'true';
			if (($value == "'false'") && ($boolAsString))
			$value = 'false';

			$attribute_values[$attrib] = $value;
		}
		return $attribute_values;
	}


	function generateObjectParamEmbedCode($object_attribute_values, $param_attribute_values, $embed_attribute_values) {

		//put together the actual embed HTML
		$strMediaEmbedCode = '<object ';
		foreach ($object_attribute_values as $k=>$v) {
			$strMediaEmbedCode .= $k.'="'.$v.'" ';
		}
		$strMediaEmbedCode .= '> ';

		foreach ($param_attribute_values as $k=>$v) {
			$strMediaEmbedCode .= '<param name="'.$k.'" value="'.$v.'" />';
		}

		$strMediaEmbedCode .= '<embed ';
		foreach ($embed_attribute_values as $k=>$v) {
			$strMediaEmbedCode .= $k.'="'.$v.'" ';
		}
		$strMediaEmbedCode .= '></embed>';
		$strMediaEmbedCode .= '</object>';

		return $strMediaEmbedCode;
	}
}
?>